from django.test import TestCase

from feedme.models import Feed, FeedItem


class FeedMeTestCase(TestCase):

    def test_update_feed(self):
        feed = Feed(url='http://derek.stegelman.com/blog/feed/')
        feed.save()
        feed.update(force=True)
        self.assertTrue(FeedItem.objects.all().count(), 10)

    def test_me(self):
        self.assertTrue(True, True)


from django.contrib import admin

from .models import Feed, FeedItem, Category


class FeedItemAdmin(admin.ModelAdmin):
    list_display = ('title', 'read')


class FeedAdmin(admin.ModelAdmin):
    list_display = ('title', 'url', 'category', 'get_unread_count', 'last_update')
    readonly_fields = ['last_update', 'title']
    save_on_top = True

admin.site.register(Feed, FeedAdmin)
admin.site.register(FeedItem, FeedItemAdmin)
admin.site.register(Category)


from django.views.generic import ListView, DetailView
from django.conf import settings

from infuse.auth.permissions import LoginRequiredMixin

from .models import Feed, FeedItem, Category
from .forms import AddFeedForm


class FeedList(LoginRequiredMixin, ListView):
    """
    Show the feed list for the logged in user.
    @todo - need to protect ti with logged in user
    """
    template_name = 'feedme/feed_list.html'
    context_object_name = 'feed_items'

    def update_feeds(self, user):
        for feed in Feed.objects.filter(user=user):
            feed.update()
        return True

    def get_queryset(self):
        # Update the feed on page load..
        self.update_feeds(self.request.user)
        items = FeedItem.objects.my_feed_items(self.request.user).un_read()

        if self.kwargs.get('category', None):
            return items.category(self.kwargs.get('category'))

        if self.kwargs.get('feed_id', None):
            return items.filter(feed__id=self.kwargs.get('feed_id'))

        return items

    def get_context_data(self, **kwargs):
        context = super(FeedList, self).get_context_data(**kwargs)
        context['categories'] = Category.objects.all()
        context['add_form'] = AddFeedForm()

        return context




from django.conf.urls import patterns, include, url

from .views import FeedList
from .ajax import mark_as_read

urlpatterns = patterns('',
   url(r'^$', FeedList.as_view(), name="feedme-feed-list"),
   url(r'^by_category/(?P<category>[-\w]+)/$', FeedList.as_view(), name='feedme-feed-list-by-category'),
   url(r'^by_feed/(?P<feed_id>[-\w]+)/$', FeedList.as_view(), name='feedme-feed-list-by-feed'),
   url(r'^ajax/mark_as_read/$', mark_as_read, name='feedme-mark-as-read-ajax'),
   #url(r'^(?P<ethnicity>[-\w]+)/$', EateryList.as_view(), name='eatery-by-ethnicity'),
   #url(r'^eatery/(?P<slug>[-\w]+)/$', EateryDetail.as_view(), name="eatery-detail"),

   )



from django.shortcuts import get_object_or_404
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from .models import FeedItem

@csrf_exempt
def mark_as_read(request):
    if request.method == "POST":
        feed_item = get_object_or_404(FeedItem, pk=request.POST.get('feed_item_id'))
        feed_item.read = True
        feed_item.save()

    return HttpResponse()



from setuptools import setup

setup(
    name='django-feedme',
    version='0.3',
    packages=['feedme', 'feedme.migrations'],
    zip_safe = False,
    include_package_data=True,
    url='http://github.com/dstegelman/django-feedme',
    license='MIT',
    author='Derek Stegelman',
    author_email='email@stegelman.com',
    description='Django Google Reader Replacement',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        'Programming Language :: Python :: 2.7',
        "Framework :: Django",
        ],
    install_requires = ['feedparser==5.1.3', 'django-bootstrap-static==2.3.1', 'hadrian==1.1.5']
    )


import datetime

from django.db import models
from django.contrib.auth.models import User
from django.conf import settings

from hadrian.utils.slugs import unique_slugify

import feedparser

from .managers import FeedItemManager


class Category(models.Model):
    name = models.CharField(max_length=250, blank=True)
    slug = models.SlugField(blank=True, null=True, editable=False)
    user = models.ForeignKey(User, blank=True, null=True)

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        unique_slugify(self, self.name)
        super(Category, self).save(*args, **kwargs)

    @property
    def get_unread_count(self):
        return FeedItem.objects.my_feed_items(self.user).category(self.slug).un_read().count()


class Feed(models.Model):
    url = models.CharField(blank=True, max_length=450)
    title = models.CharField(blank=True, null=True, max_length=250)
    category = models.ForeignKey(Category, blank=True, null=True)
    user = models.ForeignKey(User, blank=True, null=True)
    last_update = models.DateField(blank=True, null=True)

    def __unicode__(self):
        return self.url

    def _get_title(self):
        parser = feedparser.parse(self.url)
        return parser.feed.title

    def save(self, *args, **kwargs):
        if not self.title:
            self.title = self._get_title()
        super(Feed, self).save(*args, **kwargs)
        if self.last_update is None:
            self.update(force=True)

    @property
    def get_unread_count(self):
        return FeedItem.objects.filter(feed=self).un_read().count()

    def _update_feed(self):
        # Update the last update field
        self.last_update = datetime.date.today()
        self.save()
        for item in feedparser.parse(self.url).entries[:10]:
            # Search for an existing item
            try:
                FeedItem.objects.get(title=item.title)
            except FeedItem.DoesNotExist:
                # Create it.
                feed_item = FeedItem(title=item.title, link=item.link, content=item.description, feed=self)
                feed_item.save()

    def _update_processor(self):
        if getattr(settings, 'FEED_UPDATE_CELERY', False):
            from .tasks import update_feed
            update_feed.delay(self)
            return True
        self._update_feed()
        return True

    def update(self, force=False):
        """ If we aren't forcing it
        and its not the same day, go ahead
        and update the feeds.
        """

        if force or not force and self.last_update < datetime.date.today():
            self._update_processor()
            return True

    @models.permalink
    def get_absolute_url(self):
        return ('feedme-feed-list-by-feed', (), {'feed_id': self.id})


class FeedItem(models.Model):
    title = models.CharField(max_length=350, blank=True)
    link = models.URLField(blank=True)
    content = models.TextField(blank=True)
    feed = models.ForeignKey(Feed, blank=True, null=True)
    read = models.BooleanField()

    objects = FeedItemManager()

    class Meta:
        ordering = ['id']

    def __unicode__(self):
        return self.title




# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Category'
        db.create_table(u'feedme_category', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=250, blank=True)),
        ))
        db.send_create_signal(u'feedme', ['Category'])

        # Adding model 'Feed'
        db.create_table(u'feedme_feed', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('url', self.gf('django.db.models.fields.URLField')(max_length=200, blank=True)),
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['feedme.Category'], null=True, blank=True)),
        ))
        db.send_create_signal(u'feedme', ['Feed'])

        # Adding model 'FeedItem'
        db.create_table(u'feedme_feeditem', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=350, blank=True)),
            ('link', self.gf('django.db.models.fields.URLField')(max_length=200, blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal(u'feedme', ['FeedItem'])


    def backwards(self, orm):
        # Deleting model 'Category'
        db.delete_table(u'feedme_category')

        # Deleting model 'Feed'
        db.delete_table(u'feedme_feed')

        # Deleting model 'FeedItem'
        db.delete_table(u'feedme_feeditem')


    models = {
        u'feedme.category': {
            'Meta': {'object_name': 'Category'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '250', 'blank': 'True'})
        },
        u'feedme.feed': {
            'Meta': {'object_name': 'Feed'},
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['feedme.Category']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'})
        },
        u'feedme.feeditem': {
            'Meta': {'object_name': 'FeedItem'},
            'content': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'link': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '350', 'blank': 'True'})
        }
    }

    complete_apps = ['feedme']


from django.db.models.query import QuerySet
from django.db.models import Manager


class FeedItemQuerySet(QuerySet):
    def my_feed_items(self, user):
        return self.filter(feed__user=user)

    def category(self, category_slug):
        return self.filter(feed__category__slug=category_slug)

    def un_read(self):
        return self.filter(read=False)

    def read(self):
        return self.filter(read=True)


class FeedItemManager(Manager):
    def get_query_set(self):
        return FeedItemQuerySet(self.model, using=self._db)

    def category(self, category_slug):
        return self.get_query_set().category(category_slug)

    def my_feed_items(self, user):
        return self.get_query_set().my_feed_items(user)

    def un_read(self):
        return self.get_query_set().un_read()

    def read(self):
        return self.get_query_set().read()





from celery.task import task

@task(name='update_feeds')
def update_feed(feed):
    feed._update_feed()


@task(name='update_all_feeds')
def update_all_feeds():
    from .models import Feed
    for feed in Feed.objects.all():
        feed._update_feed()

